@@ -10,7 +10,7 @@
 #'
 #' Check the vignette "Maraca Plots - Plotting win odds" for more details.
 #'
-#' @param x an object of S3 class 'maraca' or 'hce'.
+#' @param x an object of S3 class 'maraca' or 'adhce'.
 #' @param theme Choose theme to style the plot. The default theme is "maraca".
 #'        Options are "maraca", "color1", "color2" and none".
 #'        For more details, check the vignette called
@@ -31,14 +31,6 @@
 #'                   in this order. Note that this parameter only need to
 #'                   be specified if you have labels different from
 #'                    "active" and "control".
-#' @param fixed_followup_days Not needed if HCE object contains information
-#'                            on fixed follow-up days in the study
-#'                            (column PADY or TTEfixed,
-#'                            depending on hce version).
-#'                            Otherwise, this argument must be specified.
-#'                            Note: If argument is specified and HCE object
-#'                            contains PADY or TTEfixed column, then
-#'                            fixed_followup_days argument is used.
 #' @param theme Choose theme to style the plot. The default theme is "maraca".
 #'        Options are "maraca", "color1", "color2" and none".
 #'        For more details, check the vignette called
@@ -88,7 +80,7 @@ component_plot <- function(x, ...) {
 component_plot.default <- function(x,
                                    ...) {
   paste0("component_plot() function can only handle inputs of class ",
-         "'hce' or 'maraca'. Your input has class ", class(x), ".")
+         "'adhce' or 'maraca'. Your input has class ", class(x), ".")
 }
 
 
@@ -126,18 +118,16 @@ component_plot.maraca <- function(x,
 
 #' @rdname component_plot
 #' @export
-component_plot.hce <- function(x, step_outcomes = NULL,
-                               last_outcome = "C",
-                               arm_levels = c(active = "A", control = "P"),
-                               fixed_followup_days = NULL,
-                               theme = "maraca",
-                               lowerBetter = FALSE,
-                               ...) {
+component_plot.adhce <- function(x, step_outcomes = NULL,
+                                 last_outcome = "C",
+                                 arm_levels = c(active = "A", control = "P"),
+                                 theme = "maraca",
+                                 lowerBetter = FALSE,
+                                 ...) {
 
   # Create maraca object
   maraca_dat <- .maraca_from_hce_data(x, step_outcomes,
                                       last_outcome, arm_levels,
-                                      fixed_followup_days,
                                       compute_win_odds = TRUE,
                                       lowerBetter = lowerBetter)
 
@@ -165,7 +155,7 @@ component_plot.hce <- function(x, step_outcomes = NULL,
 #' Generic function to create a plot showing the components used in
 #' calculating win odds (wins and ties) cumulated for all
 #' outcomes for a hierarchical endpoint.
-#' Implemented for objects of type 'maraca' and 'hce'.
+#' Implemented for objects of type 'maraca' and 'adhce'.
 #'
 #' Note that for this plot, if applying to a maraca object, in the
 #' original maraca() function run the argument "compute_win_odds" has
@@ -173,7 +163,7 @@ component_plot.hce <- function(x, step_outcomes = NULL,
 #'
 #' Check the vignette "Maraca Plots - Plotting win odds" for more details.
 #'
-#' @param x an object of S3 class 'maraca' or 'hce'.
+#' @param x an object of S3 class 'maraca' or 'adhce'.
 #' @param theme Choose theme to style the plot. The default theme is "maraca".
 #'        Options are "maraca", "color1", "color2" and none".
 #'        For more details, check the vignette called
@@ -200,14 +190,6 @@ component_plot.hce <- function(x, step_outcomes = NULL,
 #'                   in this order. Note that this parameter only need to
 #'                   be specified if you have labels different from
 #'                    "active" and "control".
-#' @param fixed_followup_days Not needed if HCE object contains information
-#'                            on fixed follow-up days in the study
-#'                            (column PADY or TTEfixed,
-#'                            depending on hce version).
-#'                            Otherwise, this argument must be specified.
-#'                            Note: If argument is specified and HCE object
-#'                            contains PADY or TTEfixed column, then
-#'                            fixed_followup_days argument is used.
 #' @param lowerBetter Flag for the final outcome variable, indicating if
 #'                    lower values are considered better/advantageous.
 #'                    This flag is need to make sure the win odds are
@@ -261,7 +243,7 @@ dustin_plot <- cumulative_plot
 #' @export
 cumulative_plot.default <- function(x, ...) {
   paste0("cumulative_plot() function can only handle inputs of class ",
-         "'hce' or 'maraca'. Your input has class ", class(x), ".")
+         "'adhce' or 'maraca'. Your input has class ", class(x), ".")
 }
 
 #' @rdname cumulative_plot
@@ -301,20 +283,18 @@ cumulative_plot.maraca <- function(x, theme = "maraca",
 
 #' @rdname cumulative_plot
 #' @export
-cumulative_plot.hce <- function(x, step_outcomes = NULL,
-                                last_outcome = "C",
-                                arm_levels = c(active = "A", control = "P"),
-                                fixed_followup_days = NULL,
-                                theme = "maraca",
-                                include = c("win odds", "win ratio"),
-                                reverse = FALSE,
-                                lowerBetter = FALSE,
-                                ...) {
+cumulative_plot.adhce <- function(x, step_outcomes = NULL,
+                                  last_outcome = "C",
+                                  arm_levels = c(active = "A", control = "P"),
+                                  theme = "maraca",
+                                  include = c("win odds", "win ratio"),
+                                  reverse = FALSE,
+                                  lowerBetter = FALSE,
+                                  ...) {
 
   # Create maraca object
   maraca_dat <- .maraca_from_hce_data(x, step_outcomes,
                                       last_outcome, arm_levels,
-                                      fixed_followup_days,
                                       compute_win_odds = TRUE,
                                       lowerBetter = lowerBetter)
 
